;;
;; HD-Bug U.L.S. - RC2.0a (WHDload for the Atari ST Series)
;; Possible way to convert DMA loaders to HD
;;
;; Cyrano Jones, Showaddywaddy, GGN and Defjam of the Megamighty D-Bug
;;

;;
;; Conditional Flags
;;

;debugging		; change colours during ULS
;mste_16mhz		; comment out for 8mhz
adv_debug		; use advanced debugging displays
byte_search		; byte-pattern matching and replacement
;autoeoi		; enable if using auto End of Interupt mode
;simple_mfp_code	; enable for original, disable for Defjam's
;force_512k		; sets Physical RAMtop to 512k
;bbc_dma_loader		; Andy The Arfling's BBC DMA fileloader
neil_dma_loader		; Neil of CB's DMA fileloader

;;
;; U.L.S. Constants (Need to change this to a pointer to a table so its dynamic)
;;

start_screen	equ $78000		; address of screen (@init)
ULS_base	equ $202000
highest_address	equ $400000		; first unaddressable byte

base_ramtop	equ ULS_base+$000	; save up to this address
shadow_tos	equ ULS_base+$004	; save TOS at this address
shadow_ram	equ ULS_base+$008	; save RAM at this address 
HD_filename	equ ULS_base+$00c	; copy filename
HD_loadadd	equ ULS_base+$020	; copy load address
HD_bytes_loaded	equ ULS_base+$024	; return length
HD_filebuffer	equ ULS_base+$028	; address of filebuffer
HD_Bug_loader	equ ULS_base+$02c	; address of dbug loader
HD_status_reg	equ ULS_base+$02e	; dump of status register
HD_status_regl	equ ULS_base+$030	; dump of executing SR
HD_filesize	equ ULS_base+$034
HD_registers	equ ULS_base+$040	; register dump
HD_MFP_LOW	equ ULS_base+$600	; MFP status dump (running app)
HD_MFP		equ ULS_base+$700	; MFP status dump (TOS)
memtop_base	equ ULS_base+$800	; where to start dumping
new_screen	equ ULS_base+$100	; my screen address
old_screen	equ ULS_base+$104	; old screen address
longstore	equ ULS_base+$108	; temp longword storage


	pea code			; execute supervisor mode
	move.w #$26,-(a7)
	trap #14
	
code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ULS Init Routines
;;
	include ULS_init.s
;;
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME SPECIFIC INIT ROUTINE
;; (This is where we set up the gameloader and patch in U.L.S.) 
;;

	lea .game2,a0
	lea $100000,a1
	lea .game3,a2
.game1	move.b (a0)+,(a1)+
	cmp.l a0,a2
	bne.s .game1
	jmp $100000

.game2	move.w #$2700,sr

	lea loader_shell,a0
	lea r_main,a1
	move.l HD_Bug_loader,a2
.game4	move.b (a0)+,(a2)+
	cmp.l a0,a1
	bne.s .game4

	lea r_main,a0
	lea $2000,a1
	lea r_main_end,a2
.game5	move.b (a0)+,(a1)+
	cmp.l a0,a2
	bne.s .game5

	lea $2000,a0
	move.l HD_Bug_loader,a2
	jsr 4(a2)

	move.w #$4ef9,$2148.w
	move.l HD_Bug_loader,$2148+2.w	

	move.b #$60,$237c
	move.b #$60,$23cc
	move.b #$60,$239c

	move.w #$0,$ec.w
	
	jmp $2000			; execute loader
.game3	illegal


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ULS CORE ROUTINES
;;
	include uls_core.s		
;;
;;

;; determines ULS read/write of current file
;;
;; d0.w=0 for read...   anything else for write 


bytes_to_write	equ 20000	; bytes to save in file if write

read_write_test
	clr.w d0
	rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ULS SEARCH ROUTINES
;;

		ifd byte_search

globalsearch
	dc.l $4cdf7fff			; do original instruction
global_entry
	movem.l d0-d7/a0-a6,-(a7)

	lea .patch1add(pc),a6
	move.l HD_Bug_loader,d6
	move.l d6,(a6)

	lea .patch4add(pc),a5
	lea globalsearch(pc),a6
	move.l a6,(a5)

	lea .pattern1(pc),a5
	lea .patch1(pc),a6
	bsr searchbytes

	lea .pattern2(pc),a5
	lea .patch2(pc),a6
	bsr searchbytes

	lea .pattern3(pc),a5
	lea .patch3(pc),a6
	bsr searchbytes

	lea .pattern4(pc),a5
	lea .patch4(pc),a6
	bsr searchbytes

	lea .pattern5(pc),a5
	lea .patch5(pc),a6
	bsr searchbytes

	lea .pattern6(pc),a5
	lea .patch6(pc),a6
	bsr searchbytes

	movem.l (a7)+,d0-d7/a0-a6
	clr.l d0
	rts


.pattern1	dc.l $47f88604,$49f88606
		dc.w $7801
		dc.w -1
.patch1		dc.w $4ef9
.patch1add	dc.l 0
		dc.w -1

.pattern2	dc.l $0c791234,$0008003c,$660e2010
		dc.w -1
.patch2		dc.l $0c791234,$0008003c,$600e2010
		dc.w -1

.pattern3	dc.l $0c791234,$0008003c,$660e2257
		dc.w -1
.patch3		dc.l $0c791234,$0008003c,$600e2257
		dc.w -1

.pattern4	dc.l $4cdf7fff,$42804e75
		dc.w -1
.patch4		dc.w $4ef9
.patch4add	dc.l 0
		dc.w -1

; patch the memory tester

.pattern5	dc.l $0cb80027,$e4700008,$67047640
		dc.w -1
.patch5		dc.l $0cb80027,$e4700008,$67047610
		dc.w -1

.pattern6	dc.l $0c791234,$0008003c,$660a2257,$20104eb9
		dc.w -1
.patch6		dc.l $0c791234,$0008003c,$600a2257
		dc.w -1	

		endc


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Binary dump of loader
;;


r_main	incbin "2000.epi"			; dump of loader 
r_main_end



