;;
;; HD_Loader Init routine
;;

HD_Bug_Init
	move.l #code,d0			; U.L.S. stub load address
	add.l #512,d0
	move.l d0,base_ramtop		; last byte to need saving

	move.l #memtop_base,d1
	move.l d1,shadow_tos		; address of TOS copy
	add.l d0,d1
	move.l d1,shadow_ram		; address of RAM copy
	add.l d0,d1
	move.l d1,HD_filebuffer		; address of FILE buffer
	move.l #r_main,d0
	sub.l #loader_shell,d0
	move.l #highest_address-2,d1
	sub.l d0,d1
	move.l d1,HD_Bug_loader		; address of HD Bug loader

	ifd adv_debug
	sub.l #64000,d1			; address of debugging info screen
	clr.b d1			; @loader address ~33k bytes
	move.l d1,new_screen
	endc

	move.w #$2700,sr		; kill everything

	lea $8,a0			; save lowmem ram
	move.l base_ramtop,a1
	move.l shadow_tos,a2
.save	move.l (a0)+,(a2)+
	cmp.l a0,a1
	bgt.s .save		

	rts

;;
;; U.L.S. Shellcode (To patch into game)
;;
loader_shell
	bra .ULS			; ULS+0 	ULS Load Code Start
	ifd byte_search
	bra global_entry		; ULS+2		Search and replace code (a0=start add)
	endc
	ifnd byte_search
	nop
	endc
.ULS
	move.w sr,-(a7)			; save SR
	stop #$2300
	move.w #$2700,sr
	movem.l d0-7/a0-7,HD_registers	; save registers

	ifd adv_debug
	move.l 2(a7),longstore
	endc 

	lea loader_shell(pc),a7		 
 	lea -$100(a7),a7		; our stack (below load code)
	move.l a7,usp

	ifd adv_debug			; advanced debugging info setup
	movem.l d0-7/a0-a6,-(a7)

	bsr info_init			; create font table relative to pc
	move.l #old_screen,a0
	move.l #new_screen,a1
	move.b $ffff8201.w,(a0)+
	move.b $ffff8203.w,(a0)		; store current screen address
	move.w #32000/4,d0
	move.l (a1),a2
.clrit	clr.l (a2)+			; erase debugging info screen
	dbra d0,.clrit
	move.b 1(a1),$ffff8201.w	; set debugging screen
	move.b 2(a1),$ffff8203.w

	lea $ffff8240.w,a0
	lea old_pal(pc),a1
	move.w #15,d0
.palt	move.w (a0),(a1)+		; store current pallete and set all planes to white
	move.w #$777,(a0)+
	dbra d0,.palt
	clr.w $ffff8240.w		; except background

	lea t_init(pc),a1
	move.l new_screen,a0
	bsr info			; print "ENTERING ULS!"

	move.l longstore,d1		; caller address
	bsr hashing			; convert to ascii
	lea hexad(pc),a1
	lea t_jmpa(pc),a0
	move.l (a1)+,(a0)+
	move.l (a1),(a0)
	lea t_jmp(pc),a1
	move.l new_screen,a0
	bsr info			; print "Called from: $xxxxxxxx"

	movem.l (a7)+,d0-7/a0-a6
	endc

;;
;; GAME/LOADER SPECIFIC READ/WRITE TEST
;;
;; In Barbarian II (Psygnosis) Hotknife sets $31.w -ve for WRITE
;;
	bsr read_write_test
	tst.w d0
	beq	.read_file

.write_file
	bsr .process_file
	bsr .bytecopy_file_up		; copy filedata to high memory
	bsr .restore_gemdos	        ; swap memory back in
	bsr .save_file			; gemdos filewrite
	bsr .restore_lowmem		; put it all back
	
	bra .gemdos_done

.read_file
	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	lea t_read(pc),a1
	move.l new_screen,a0
	bsr info			; print ULS function
	movem.l (a7)+,d0-7/a0-6
	endc

	bsr .process_file		; process filename & load address
	bsr .restore_gemdos		; swap memory back in
	bsr .load_file			; gemdos fileload
	bsr .restore_lowmem		; put it all back
	bsr .bytecopy_file		; insert file at correct address
	
.gemdos_done
	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)

	lea t_done(pc),a1
	move.l new_screen,a0
	bsr info			; print "Done!"

	move.l #$3ffff,d0
.longwait				; delay so you can actually read it
	eor.w #$7,$ffff8240.w
	subq.l #1,d0
	bne.s .longwait
	
	move.l #old_screen,a0		; restore screen
	move.b (a0)+,$ffff8201.w
	move.b (a0),$ffff8203.w

	lea old_pal(pc),a0
	lea $ffff8240.w,a1
	move.w #15,d0
.palt2	move.w (a0)+,(a1)+		; restore pallette
	dbra d0,.palt2

	movem.l (a7)+,d0-7/a0-6
	endc

	ifd simple_mfp_code
	move.b HD_MFP_LOW,$fffffa07.w
	move.b HD_MFP_LOW+1,$fffffa09.w
	move.b HD_MFP_LOW+2,$fffffa15.w
	move.b HD_MFP_LOW+3,$fffffa1b.w
	move.b HD_MFP_LOW+4,$fffffa25.w
	move.b HD_MFP_LOW+5,$fffffa23.w
	move.b HD_MFP_LOW+6,$fffffa21.w
	move.b HD_MFP_LOW+7,$fffffa1f.w
	move.b HD_MFP_LOW+8,$fffffa1d.w
	 bclr.b #5,$fffffa09.w		; <- Timer C enable (thnx DefJam)
	 bclr.b #5,$fffffa15.w		; <- Timer C enable (thnx DefJam)
	endc

	ifnd simple_mfp_code
         lea     APPta_data(PC),A3
         lea     APPta_control(PC),A4
         bsr     restore_timers
	 lea	APPmfp_save(pc),a1
         bsr     restore_mfp
;        bsr     flush_acia
	endc

	ifd debugging
	clr.w	$ffff8240.w		; *****BLACK*****
	endc
	
	move.l HD_bytes_loaded,d0
	move.l d0,$e8.w
	
	move.l HD_loadadd,a0
	lea $1fc8a,a1
	move.l (a0)+,(a1)+
	move.l (a0)+,(a1)+
	move.l (a0)+,(a1)+

	movem.l HD_registers,d0-7/a0-7	; restore registers

	rte			

;;
;;
;; Process the filename & load address
;;
;; BBC DMA loader does not contain the '.' in the filename and is in RAW 11 byte format
;; Neil's loader does contain the '.' in the filename and is in 8.3 format
;;
;; expects:
;;	a0 - pointer to filename
;;	a1 - address to load at
;;
;; *Patched for odd source address of filename
;;

.process_file
	cmp.l #$1fe38,a1
	bne.s .load_a1
	move.l a2,a1

.load_a1
	move.l a1,HD_loadadd
	lea HD_filename,a1

	rept 8
	move.b (a0)+,(a1)+		; copy filename
	endr	

	ifd bbc_dma_loader
	 move.b #'.',(a1)+	; add the '.' (bbc fileloader)
	 move.b (a0)+,(a1)+	; add extender
	 move.b (a0)+,(a1)+
	 move.b (a0)+,(a1)
	endc 

	ifd neil_dma_loader
	 rept 4
	 move.b (a0)+,(a1)+	; copy the '.'+ext
	 endr		
	endc

	clr.b HD_filename+12		; terminate with 0

	lea HD_filename,a1
	move.w #11,d0
.fix	cmp.b #'.',(a1)+
	bne.s .nodot
	clr.b -1(a1)
.nodot	dbra d0,.fix

	ifd adv_debug
	movem.l d0-d7/a0-a6,-(a7)

	lea t_proc(pc),a1
	lea prt_fn(pc),a0
	clr.l (a0)
	clr.l 4(a0)
	clr.l 8(a0)
	lea HD_filename,a2
.fncopy
	move.b (a2)+,d0
	beq.s .fncopdone
	move.b d0,(a0)+
	bra.s .fncopy
.fncopdone
	move.b #-1,(a0)
	move.l new_screen,a0
	bsr info			; print filename
	movem.l (a7)+,d0-d7/a0-a6
	endc

	rts

;;
;;
;; Restore GEMDOS (& possibly the MFP if title requires it)
;;
;;

.restore_gemdos
	move.w	#$2700,sr		; kill all timers

	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	lea t_rgem(pc),a1
	move.l new_screen,a0
	bsr info
	movem.l (a7)+,d0-7/a0-6
	endc

	ifd debugging
	move.w #$700,$ffff8240.w	; *****RED*****
	endc

	lea $8,a0			; save lowmem ram
	move.l base_ramtop,a1
	move.l shadow_ram,a2
.save	move.l (a0)+,(a2)+
	cmp.l a0,a1
	bgt.s .save		

	ifd simple_mfp_code
	 move.b $fffffa07.w,HD_MFP_LOW	
	 move.b $fffffa09.w,HD_MFP_LOW+1
	 move.b $fffffa15.w,HD_MFP_LOW+2
	 move.b $fffffa1b.w,HD_MFP_LOW+3
	 move.b $fffffa25.w,HD_MFP_LOW+4
	 move.b $fffffa23.w,HD_MFP_LOW+5
	 move.b $fffffa21.w,HD_MFP_LOW+6
	 move.b $fffffa1f.w,HD_MFP_LOW+7
	 move.b $fffffa1d.w,HD_MFP_LOW+8

	 move.b HD_MFP,$fffffa07.w
	 move.b HD_MFP+1,$fffffa09.w
	 move.b HD_MFP+2,$fffffa15.w
	 move.b HD_MFP+3,$fffffa1b.w
	 move.b HD_MFP+4,$fffffa25.w
	 move.b HD_MFP+5,$fffffa23.w
	 move.b HD_MFP+6,$fffffa21.w
	 move.b HD_MFP+7,$fffffa1f.w
	 move.b HD_MFP+8,$fffffa1d.w

	 bset.b #5,$fffffa09.w		; <- Timer C enable (thnx DefJam)
	 bset.b #5,$fffffa15.w		; <- Timer C enable (thnx DefJam)
	 move.b #$c0,$fffffa23.w	; <- Timer C data (thnx Grazey)
	endc

	ifnd simple_mfp_code
	 lea	APPmfp_save(pc),a1
         bsr     save_mfp
         lea     APPta_control(PC),A0
         lea     APPta_data(PC),A3
         bsr     save_timers
         move    #$2700,SR
         lea     GEMta_data(PC),A3
         lea     GEMta_control(PC),A4
         bsr     restore_timers
	 lea	GEMmfp_save(pc),a1
         bsr     restore_mfp
;        bsr     flush_acia
	endc

; if not in auto-eoi mode, the interrupt-in service registers have to be
; cleared !
	ifd autoeoi
        clr.b   $FFFFFA0F.w
        clr.b   $FFFFFA11.w
	endc

	ifd debugging
	move.w #$70,$ffff8240.w		; *****GREEN*****
	endc
	
	move.l shadow_tos,a0		; move GEM lowmemory back
	lea $8,a1
	move.l shadow_ram,a2
.rest	move.l (a0)+,(a1)+
	cmp.l a0,a2
	bgt.s .rest
	rts

;;
;;
;; GEMDos trap #1 load the file
;;
;;

.load_file
	move.w #$2300,sr
	
	ifd debugging
	move.w #$777,$ffff8240.w	; *****WHITE*****
	endc

	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	lea t_3d(pc),a1
	move.l new_screen,a0
	bsr info			; print Fopen
	movem.l (a7)+,d0-7/a0-6
	endc


	clr.w -(a7)			; open file
	move.l #HD_filename,-(a7)
	move.w #$3d,-(a7)		; GEMDOS fopen
	trap #1
	add.l #8,a7
	move.w d0,d7			; file handle
	bmi.s	.crud			; loop if file not found

	ifd debugging
	sub.w #$333,$ffff8240.w
	endc

	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	move.l HD_filebuffer,d1
	bsr hashing
	lea hexad(pc),a1
	lea t_3fa(pc),a0
	move.l (a1)+,(a0)+
	move.l (a1),(a0)
	move.l HD_loadadd,d1
	bsr hashing
	lea hexad(pc),a1
	lea t_3fb(pc),a0
	move.l (a1)+,(a0)+
	move.l (a1),(a0)
	lea t_3f(pc),a1
	move.l new_screen,a0
	bsr info			; print load address
	movem.l (a7)+,d0-7/a0-6
	endc

	move.l HD_filebuffer,-(a7)	; read file into high memory (in case it wanted to load low)
	move.l #$7ffff,-(a7)
	move.w d7,-(a7)
	move.w #$3f,-(a7)		; GEMDOS fread
	trap #1
	lea 12(a7),a7
	move.l d0,d6
	bpl.s .okyeah
.crud	addq.w #1,$ffff8240.w		; loop here on error (read failed)
	bra.s .crud
.okyeah	move.l d6,HD_bytes_loaded	; save bytes loaded counter
	movem.l d6/a0,-(a7)
	move.l HD_filebuffer,a0
	cmp.l #'ATM5',(a0)
	bne .xdone2
	move.l 4(a0),HD_filesize
	bra.s .xd1
.xdone2	move.l d6,HD_filesize		; save bytes loaded counter
.xd1	movem.l (a7)+,d6/a0

	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	move.l HD_bytes_loaded,d1
	bsr hashing
	lea hexad(pc),a1
	lea t_3fda(pc),a0
	move.l (a1)+,(a0)+
	move.l (a1),(a0)
	lea t_3fd(pc),a1
	move.l new_screen,a0
	bsr info			; print bytes loaded
	movem.l (a7)+,d0-7/a0-6
	endc


	ifd debugging
	sub.w #$333,$ffff8240.w
	endc

	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	lea t_40(pc),a1
	move.l new_screen,a0
	bsr info			; print Closed
	movem.l (a7)+,d0-7/a0-6
	endc

	move.w d7,-(a7)			
	move.w #$3e,-(a7)		; GEMDOS fclose
	trap #1
	addq.l #4,a7

	rts

;;
;;
;; GEMDos trap #1 save the file
;;
;;


.save_file
	move.w #$2300,sr
	
	ifd debugging
	move.w #$777,$ffff8240.w	; *****WHITE*****
	endc

	clr.w -(a7)			; open file
	move.l #HD_filename,-(a7)
	move.w #$3c,-(a7)		; GEMDOS fwrite
	trap #1
	add.l #8,a7
	move.w d0,d7			; file handle
	bmi.s	.crud1			; loop if create error

	ifd debugging
	sub.w #$333,$ffff8240.w
	endc

	move.l HD_filebuffer,-(a7)	; read file into high memory (in case it wanted to load low)
	move.l #bytes_to_write,-(a7)		; ************** Bytes to write *************************** HARDCODED *********
	move.w d7,-(a7)
	move.w #$40,-(a7)		; GEMDOS fwrite
	trap #1
	lea 12(a7),a7
	move.l d0,d6
	bpl.s .okyeh
.crud1	addq.w #1,$ffff8240.w		; loop here on error (read failed)
	bra.s .crud1
.okyeh	move.l d6,HD_bytes_loaded	; save bytes loaded counter

	ifd debugging
	sub.w #$333,$ffff8240.w
	endc

	move.w d7,-(a7)			
	move.w #$3e,-(a7)		; GEMDOS fclose
	trap #1
	addq.l #4,a7

	rts

;;
;;
;; Put original low memory back
;;
;;

.restore_lowmem
	ifd adv_debug
	movem.l d0-7/a0-6,-(a7)
	lea t_low(pc),a1
	move.l new_screen,a0
	bsr info
	movem.l (a7)+,d0-7/a0-6
	endc


	ifd debugging
	move.w #$7,$ffff8240.w		; ******BLUE*****
	endc

	move.w #$2700,sr		; kill timers

	move.l shadow_ram,a0		; put lowmem back
	lea $8,a1
	move.l base_ramtop,a2
.back	move.l (a0)+,(a1)+
	cmp.l a1,a2
	bgt.s .back
	rts

;;
;;
;; Bytecopy the loaded file to correct address (allows loading to any address)
;;
;;

.bytecopy_file
	ifd debugging
	move.w #$77,$ffff8240.w		; *****CYAN*****
	endc

	move.l HD_bytes_loaded,d6
	move.l HD_filebuffer,a0		; address we traploaded file at
	move.l HD_loadadd,a1		; address file required to be loaded at
	move.l a1,d0
	cmp.l #$100000,d0		; is it trying to load above 512k?
	blt.s	.fload
.feck	neg.w $ffff8240.w		; yes - attempt to load too high (neg background loop)
	bra.s .feck

.fload	move.b (a0)+,(a1)+		; byte copy the file 		
	subq.l #1,d6
	bne.s .fload

	ifd debugging
	move.w #$707,$ffff8240.w	; *****PURPLE*****
	endc

	rts

;;
;;
;; Bytecopy the save-area to protected high memory address (allows GEM to overwrite data to be saved)
;;
;;

.bytecopy_file_up
	ifd debugging
	move.w #$77,$ffff8240.w		; *****CYAN*****
	endc

	move.l HD_filebuffer,a0		; address to move data to
	move.l HD_loadadd,a1		; address file required to be saved at

	move.l #bytes_to_write-1,d6		; ************** Bytes to copy *************************** HARDCODED *********

.fwrit	move.b (a1)+,(a0)+		; byte copy the file 		
	subq.l #1,d6
	bne.s .fwrit

	ifd debugging
	move.w #$707,$ffff8240.w	; *****PURPLE*****
	endc

	rts

; calls
; 
; a0-start
; a5-pattern
; a6-replace

		ifd byte_search

searchbytes
		movem.l a0/a5/a6,-(a7)

.find		move.w (a5),d0			; get 1st word
.findloop	cmp.w (a0)+,d0			; match?
		beq	.match1			;->branch if yes
		cmp.l #$80000,a0		; reached memtop
		bne.s .findloop			; loop around

		movem.l (a7)+,a0/a5/a6		; return back
		rts

.match1		lea -2(a0),a1			; start of sequence
		move.l a5,-(a7)			; save pattern start
		lea 2(a5),a5			; correct for 1st word

.match2		cmp.w (a0)+,(a5)+		; check for match
		beq.s .match2			; keep looping if match

		cmp.l a6,a5			; end of sequence?
		beq .exact			; exact match found

		move.l (a7)+,a5			; restore pattern start
		bra .find			; keep searching

.exact		move.l (a7)+,a5

		move.l a6,-(a7)
.exactcopy	move.w (a6)+,(a1)+
		cmp.w #-1,(a6)
		bne.s .exactcopy
		move.l (a7)+,a6
		bra	.find

		endc

;
; properly saving/restoring of the timer data registers
; (hopefully)
;
; (w)Defjam/Checkpoint, September 2005
; defjam_cp@gmx.net
;

                OPT D+

;-------------------------------------------------------------------------------
flush_acia:                             ;PART
                move.w  D0,-(SP)
flush_acia_loop:
                btst    #0,$FFFFFC00.w
                beq.s   acia_empty
                move.b  $FFFFFC02.w,D0
                bra.s   flush_acia_loop
acia_empty:
                move.w  (SP)+,D0
                rts
;ENDPART
;-------------------------------------------------------------------------------
save_mfp:                               ;PART
                lea     mfp_list(PC),A0
save_mfp_loop:
                movea.w (A0)+,A2
                move.b  (A2),(A1)+
                tst.w   (A0)
                bne.s   save_mfp_loop
                rts
;ENDPART
restore_mfp:                            ;PART
                lea     mfp_list(PC),A0
restore_mfp_loop:
                movea.w (A0)+,A2
                move.b  (A1)+,(A2)
                tst.w   (A0)
                bne.s   restore_mfp_loop
                rts
;ENDPART
mfp_list:                               ;PART
                DC.W $FA07
                DC.W $FA09
                DC.W $FA13
                DC.W $FA15
                DC.W $FA17
                DC.W 0
;ENDPART
;-------------------------------------------------------------------------------
;----------------------------------------
save_timers:                            ;PART
; save timer control registers

                move.b  $FFFFFA19.w,(A0)+ ; ta_control
                move.b  $FFFFFA1B.w,(A0)+ ; tb_control
                move.b  $FFFFFA1D.w,(A0)+ ; tcd_control

; timers stop
                clr.b   $FFFFFA19.w
                clr.b   $FFFFFA1B.w
                clr.b   $FFFFFA1D.w

                lea     timer_list(PC),A2
timer_save_loop:
                move.w  #$FA00,D0
                move.b  (A2)+,D0        ; tX_data
                movea.w D0,A0
                move.b  (A2)+,D0        ; tX_ctrl
                movea.w D0,A1
                move.b  (A2)+,D0        ; tX_ctrlSet
                bsr.s   evaluate_timer
                move.b  D0,(A3)+

                tst.b   (A2)
                bne.s   timer_save_loop
                rts
;ENDPART
restore_timers:                         ;PART

; timers stop
                clr.b   $FFFFFA19.w
                clr.b   $FFFFFA1B.w
                clr.b   $FFFFFA1D.w

; restore timer_data
                lea     timer_list(PC),A2
timer_data_restore:
                move.w  #$FA00,D0
                move.b  (A2)+,D0        ; tX_data
                movea.w D0,A0
                addq.l  #2,A2           ; tX_ctrl, tX_ctrlSet

                move.b  (A3)+,D0        ; saved timer_data
                move.b  D0,(A0)
                tst.b   (A2)
                bne.s   timer_data_restore

; restore timer_control
                move.b  (A4)+,$FFFFFA19.w
                move.b  (A4)+,$FFFFFA1B.w
                move.b  (A4)+,$FFFFFA1D.w
                rts
;ENDPART

timer_list:                             ;PART
                DC.B $1F,$19,%00000101 ; ta_data, ta_ctrl, ta_ctrlSet 1:64
                DC.B $21,$1B,%00000101 ; tb_data, tb_ctrl, tb_ctrlSet 1:64
                DC.B $23,$1D,%01010000 ; tc_data, tc_ctrl, tc_ctrlSet 1:64
                DC.B $25,$1D,%00000101 ; td_data, td_ctrl, td_ctrlSet 1:64
                DC.B 0
                EVEN
;ENDPART
;----------------------------------------
; IN:
;    A0 ... timer data register
;    A1 ... timer control register
;    D0 ... timer control set
; OUT:
;    D0 ... inital timer_data
;----------------------------------------
evaluate_timer:                         ;PART

; how to evaluate the timer data setting:
;     Stop all timers ! (clear timer control registers)

; (1) wait until timer_data == 1
; (2) wait until timer_data != 1
; --> timer_data is now the initial setting of the timer data register

; PROBLEM: timer_data = 1  --> timeout counter needed

                move.w  #$1FFF,D1       ; timeout counter (if timer_data==1 !!)
                move.b  D0,(A1)         ; timer control: start
et_sync1:
                move.b  (A0),D0
                subq.b  #1,D0
                bne.s   et_sync1
;-----------------------------------------------
; timer_data is now 1
; read until timer_data!=1
;-----------------------------------------------
et_sync_reload:
                move.b  (A0),D0
                subq.b  #1,D0
                dbne    D1,et_sync_reload

                addq.b  #1,D0
                clr.b   (A1)            ; timer control: stop
                rts
;ENDPART
;----------------------------------------
GEMta_control:     DS.B 1
GEMtb_control:     DS.B 1
GEMtcd_control:    DS.B 1
GEMta_data:        DS.B 1
GEMtb_data:        DS.B 1
GEMtc_data:        DS.B 1
GEMtd_data:        DS.B 1
                EVEN
GEMmfp_save:       DS.B 64

APPta_control:     DS.B 1
APPtb_control:     DS.B 1
APPtcd_control:    DS.B 1
APPta_data:        DS.B 1
APPtb_data:        DS.B 1
APPtc_data:        DS.B 1
APPtd_data:        DS.B 1
                EVEN
APPmfp_save:       DS.B 64

	ifd adv_debug 

next_add	dc.l 0
n1		dc.l 1
n2		dc.l 7
old_pal		ds.w 16

;
; Text plotting routine
;
; call with a1 pointing at text
;           a0 pointing at plotting address

info	add.l (a1)+,a0
	lea next_add(pc),a3
	move.l a0,(a3)
	move.l #1,4(a3)		; n1
	move.l #7,8(a3)		; n2
.loop	moveq.l #0,d0
	move.b (a1)+,d0
	bpl.s .write
	cmp.b #-1,d0
	beq.s .wrap
.nextline
	move.l #1,4(a3)		; n1
	move.l #7,8(a3)		; n2
	move.l (a3),a0
	add.l #160*9,a0
	move.l a0,(a3)
	bra.s .loop
.wrap	rts
.write	sub.b #' ',d0
	add d0,d0
	add d0,d0
	lea ascii(pc),a2
	move.l (a2,d0),a2
	bsr.s .plotit
	bra. .loop
.plotit	
x	set 0
y	set 0
	rept 8
	move.b x(a2),y(a0)
x	set x+40
y	set y+160
	endr
	add.l 4(a3),a0
	move.l 4(a3),-(a7)
	move.l 8(a3),4(a3)
	move.l (a7)+,8(a3)
	rts

;	dc.b "0123456789012345678901234567890123456789"

t_init	dc.l 000*160
	dc.b "ENTERING ULS!",-1
	even

t_read	dc.l 010*160
	dc.b "ULS FUNCTION: READ",-1
	even

t_save	dc.l 010*160
	dc.b "ULS FUNCTION: WRITE",-1
	even

t_proc	dc.l 020*160
	dc.b "PROCESSING FILE: ",-2
prt_fn	ds.b 12
	dc.b -1
	even

t_rgem	dc.l 040*160
	dc.b "RESTORING GEMDOS",-1
	even

t_3d	dc.l 050*160
	dc.b "OPENING FILE",-1
	even

t_jmp	dc.l (000*160)+80
	dc.b "CALLED FROM:     ",-2
t_jmpa	ds.b 8
	dc.b -1

t_3f	dc.l 060*160
	dc.b "READING FILE AT: ",-2
t_3fa	ds.b 8
	dc.b "   ("
t_3fb	ds.b 8
	dc.b ")"
	dc.b -1
	even

t_3fd	dc.l 080*160
	dc.b "NUMBER OF BYTES: ",-2
t_3fda	ds.b 8
	dc.b -1
	even

t_40	dc.l 100*160
	dc.b "CLOSING FILE",-1
	even

t_low	dc.l 110*160
	dc.b "RESTORING LOW MEMORY",-1
	even	

t_done	dc.l 192*160
	dc.b "ALL DONE- RETURNING TO CALLER!",-1
	even
	
font_x	incbin rawfont.bin
	even

info_init
	lea font_x(pc),a0
	move.l a0,d0
	move.l d0,d1
	add.l #40*8,d1
	lea ascii(pc),a1
	move.w #39,d2
.adder	move.l d0,(a1)
	move.l d1,160(a1)
	addq.l #1,d0
	addq.l #1,d1
	lea 4(a1),a1
	dbra d2,.adder
	rts

ascii	ds.l 80

; call with long to convert in d1
; string placed at hexad
;

hashing	moveq #0,d0
	lea .table(pc),a0
	lea hexad+8(pc),a2
	bsr.s .first
	moveq.w #5,d3
.loop	bsr.s .next
	dbra d3,.loop
.next	ror.l #4,d1
.first	move.b d1,d0
	and #$f,d0
	move.b (a0,d0),-(a2)
	rts

.table	dc.b "0123456789ABCDEF"
hexad	ds.l 2
	dc.l $ffffffff
	even

	endc

loader_shell_end				; end of shellcode
	dc.b "2005 D-Bug!"	
	even